@@ -1,18 +1,17 @@
 #include <assert.h>
 
-#include "stack_ptr.hpp"
 #include "cache-t.hpp"
 
 namespace acommon {
 
+static Mutex global_cache_lock;
 static GlobalCacheBase * first_cache = 0;
-Mutex GlobalCacheBase::global_cache_lock;
 
 void Cacheable::copy() const
 {
   //CERR << "COPY\n";
   LOCK(&cache->lock);
-  copy_no_lock();
+  refcount++;
 }
 
 void GlobalCacheBase::del(Cacheable * n)
@@ -87,7 +86,7 @@ GlobalCacheBase::~GlobalCacheBase()
 
 bool reset_cache(const char * which)
 {
-  LOCK(&GlobalCacheBase::global_cache_lock);
+  LOCK(&global_cache_lock);
   bool any = false;
   for (GlobalCacheBase * i = first_cache; i; i = i->next)
   {
