@@ -791,7 +791,8 @@ namespace {
     // Read in Wordlist
     //
     {
-      const char * w0;
+      WordListIterator wl_itr(els, &lang, config.retrieve_bool("warn") ? &CERR : 0);
+      wl_itr.init(config);
       ObjStack exp_buf;
       WordAff * exp_list;
       WordAff single;
@@ -799,38 +800,25 @@ namespace {
       Vector<WordAff> af_list;
       WordData * * prev = &first;
 
-      while ( (w0 = els->next()) != 0) {
+      for (;;) {
 
-	unsigned int s = strlen(w0);
-	CharVector tstr;
-	tstr.append(w0, s+1);
-	RET_ON_ERR_SET(iconv(tstr.data(), tstr.size()), char *, w);
-        s = strlen(w);
+        PosibErr<bool> pe = wl_itr.adv();
+        if (pe.has_err()) return pe;
+        if (!pe.data) break;
 
-        if (s > 240)
-          return make_err(invalid_word, MsgConv(lang)(w),
-                          _("The total word length is larger than 240 characters."));
+        const char * w = wl_itr->word.str;
+        unsigned int s = wl_itr->word.size;
 
-        char * p0 = strchr(w, '/');
+        const char * affixes = wl_itr->aff.str;
 
-        if (p0) { // affix info
-	  s = p0 - w;
-          *p0 = '\0';
-          ++p0;
-        }
-
-        const char * affixes = p0;
-
-	RET_ON_ERR(check_if_valid(lang,w));
-
-        if (affixes && !lang.affix())
+        if (*affixes && !lang.affix())
           return make_err(other_error, 
                           _("Affix flags found in word but no affix file given."));
 
-        if (affixes && !affix_compress) {
+        if (*affixes && !affix_compress) {
           exp_buf.reset();
           exp_list = lang.affix()->expand(w, affixes, exp_buf);
-        } else if (affixes && partially_expand) {
+        } else if (*affixes && partially_expand) {
           // expand any affixes which will effect the first
           // 3 letters of a word.  This is needed so that the
           // jump tables will function correctly
@@ -839,7 +827,7 @@ namespace {
         } else {
           single.word.str = w;
           single.word.size = strlen(w);
-          single.aff = (const byte *)(affixes ? affixes : "");
+          single.aff = (const byte *)affixes;
           exp_list = &single;
         }
 
@@ -989,7 +977,9 @@ namespace {
     data.write32(0);
     unsigned prev_w_pos = data.size();
 
-    WordLookup lookup;
+    WordLookup lookup(affix_compress 
+                      ? uniq_entries * 3 / 2 
+                      : uniq_entries * 5 / 4);
     lookup.parms().block_begin = data.begin();
     lookup.parms().hash .lang     = &lang;
     lookup.parms().equal.cmp.lang = &lang;
@@ -1103,7 +1093,6 @@ namespace {
     
     //CERR.printf("%d == %d\n", lookup.size(), uniq_entries);
     //assert(lookup.size() == uniq_entries);
-    lookup.resize(lookup.size() * 5 / 4);
 
     data_head.word_count   = num_entries;
     data_head.word_buckets = lookup.bucket_count();
