@@ -229,11 +229,15 @@ static const PossibleOption * find_option(const char * str) {
   return i;
 }
 
+Conv dconv;
+Conv uiconv;
+
 int main (int argc, const char *argv[]) 
 {
 #ifdef USE_LOCALE
   setlocale (LC_ALL, "");
 #endif
+  aspell_gettext_init();
 
   options->set_committed_state(false);
 
@@ -478,6 +482,37 @@ static Convert * setup_conv(Config * config,
   }
 }
 
+void setup_display_conv()
+{
+  const char * gettext_enc = 0;
+  const char * env_enc = 0;
+  String doc_enc = options->retrieve("encoding");
+  String enc;
+#ifdef ENABLE_NLS
+  gettext_enc = bind_textdomain_codeset("aspell", 0);
+  if (gettext_enc && is_ascii_enc(gettext_enc)) gettext_enc = 0;
+#endif
+#ifdef HAVE_LANGINFO_CODESET
+  env_enc = nl_langinfo(CODESET);
+  if (is_ascii_enc(env_enc)) env_enc = 0;
+#endif
+  if (gettext_enc && env_enc && strcmp(gettext_enc,env_enc) != 0) 
+  {
+    fputs(("Error: bind_textdomain_codeset != nl_langinfo(CODESET)\n"), stderr);
+    exit(-1);
+  }
+  if (gettext_enc)
+    enc = gettext_enc;
+  else if (env_enc)
+    enc = env_enc;
+  else
+    enc = doc_enc;
+
+  fprintf(stderr, "'%s' '%s' '%s' '%s'\n", gettext_enc, env_enc, doc_enc.str(), enc.str());
+  EXIT_ON_ERR(dconv.setup(*options, doc_enc, enc, NormNone));
+  EXIT_ON_ERR(uiconv.setup(*options, enc, doc_enc, NormNone));
+}
+
 
 ///////////////////////////
 //
@@ -879,6 +914,8 @@ struct Mapping {
 
 void abort_check();
 
+void setup_display_conv();
+
 void check()
 {
   String file_name;
@@ -937,6 +974,9 @@ void check()
     print_error(aspell_error_message(ret));
     exit(1);
   }
+
+  setup_display_conv();
+
   AspellSpeller * speller = to_aspell_speller(ret);
 
   state = new CheckerString(speller,in,out,64);
@@ -962,7 +1002,7 @@ void check()
 
   while (state->next_misspelling()) {
 
-    char * word = state->get_word(word0);
+    char * word = state->get_real_word(word0);
 
     //
     // check if it is in the replace list
@@ -1027,7 +1067,7 @@ void check()
     case Exit:
       goto exit_loop;
     case Abort:
-      prompt(_("Are you sure you want to abort? "));
+      prompt(_("Are you sure you want to abort (y/n)? "));
       get_choice(choice);
       if (choice == 'y' || choice == 'Y')
         goto abort_loop;
@@ -1201,7 +1241,7 @@ void list()
  
   while (state->next_misspelling()) {
 
-    state->get_word(word);
+    state->get_real_word(word);
     COUT.printl(word);
 
   }
@@ -1839,12 +1879,14 @@ static const char * help_text[] =
   N_("  conv <from> <to> [<norm-form>]"),
   N_("    converts from one encoding to another"),
   N_("  norm (<norm-map> | <from> <norm-map> <to>) [<norm-form>]"),
-  N_("    perform unicode normlization"),
+  N_("    perform Unicode normalization"),
   usage_text[8],
   usage_text[9],
   N_("  dump|create|merge master|personal|repl [word list]"),
   N_("    dumps, creates or merges a master, personal, or replacement word list."),
   "",
+  /* TRANSLATORS: "none", "internal" and "strict" are literal values
+     and should not be translated. */
   N_("  <norm-form>      normalization form to use, either none, internal, or strict"),
   "",
   usage_text[10],
@@ -1890,7 +1932,7 @@ void print_help (bool verbose) {
     putchar('\n');
     puts(
       _("Available Dictionaries:\n"
-        "    Dictionaries can be selected directory via the \"-d\" or \"master\"\n"
+        "    Dictionaries can be selected directly via the \"-d\" or \"master\"\n"
         "    option.  They can also be selected indirectly via the \"lang\",\n"
         "    \"variety\", and \"size\" options.\n"));
     
@@ -1930,8 +1972,23 @@ void print_help (bool verbose) {
     }
 
     //
-    EXIT_ON_ERR(print_mode_help(options,COUT));
-
+    putchar('\n');
+    putchar('\n');
+    puts(  
+      /* TRANSLATORS: This should be formated to fit in 80 column or less */
+      _("Available Filter Modes:\n"
+        "    Filter Modes are reconfigured combinations of filters optimized for\n"
+        "    files of a specific type. A mode is selected via the \"mode\" option.\n"
+        "    This will happen implicitly if Aspell is able to identify the file\n"
+        "    type from the extension, and possibility the contents, of the file.\n"));
+    
+    EXIT_ON_ERR_SET(available_filter_modes(options), StringPairEnumeration *, els);
+    StringPair sp;
+    while (!els->at_end()) {
+      sp = els->next();
+      printf("  %-14s %s\n", sp.first, gt_(sp.second));
+    }
+    delete els;
   }
 }
 
